/*Autor: Kilian Httenhofer
Version: 13.08.08

einfaches Programm zur steuerung eines Belichters und hnliches

Fr atmega tiny 2313 @ 4Mhz 
*/


#include <stdlib.h>
#include "belichter.h"
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <inttypes.h>



#define KEYPORT PORTD
#define KEYDDR  DDRD
#define KEYPIN  PIND
#define abbruch PD0
#define minus  	PD1
#define plus    PD2
#define manuel  PD3
#define druck   PD4
#define lack    PD5
#define platte  PD6



#define LEDPORT	PORTB
#define LEDDDR	DDRB
#define	LED0	PB0
#define	LED1	PB1
#define	LED2	PB2
#define	LED3	PB3
#define	UVLEDS	PB4
#define schw 			135				//Schwelle ab dem "berhrung" erkannt wird
#define dyn 			10				//zeitvernderung beim drcken von plus und minus in sec
#define lackzeit		12				//Zeit fr belichten von Ltstoplack in sec
#define plattezeit		8				//zeit fr belichten von Platinen in sec
#define druckzeit		4				//Zeit fr belichten von Bestckungsdruck in sec
#define korr			15				//korrektur fr timer in sec. unmrechnen

u8 status =0;
u16 zeit =0;

void t1_init( void )
{
	TCCR0B |= (1<<CS00)|(1<<CS02);
	TIMSK = 1<<TOIE1^1<<TOIE0;
}


u8 lackcap()
{
  cli();
  u16 i = 0;
  u16 l = 0;
  KEYDDR &= ~(1<<(lack));

for (l=0;l<(schw+400);l++){
  if( KEYPIN & 1<<(lack) ){
    i++;
	}
}

  KEYPORT &= ~(1<<(lack));						
  KEYDDR |= 1<<(lack);                        	
  sei();

  return i;
}

u8 abbruchcap()
{
  cli();
  u16 i = 0;
  u16 l= 0;
  KEYDDR &= ~(1<<(abbruch));

for (l=0;l<schw+400;l++){
  if( KEYPIN & 1<<(abbruch) )
    i++;
}

  KEYPORT &= ~(1<<(abbruch));						
  KEYDDR |= 1<<(abbruch);                        	
  sei();

  return i;
}

u8 pluscap()
{
  cli();
  u16 i = 0;
  u16 l= 0;
  KEYDDR &= ~(1<<(plus));

for (l=0;l<schw+400;l++){
  if( KEYPIN & 1<<(plus) )
    i++;
}

  KEYPORT &= ~(1<<(plus));						
  KEYDDR |= 1<<(plus);                        	
  sei();

  return i;
}

u8 minuscap()
{
  cli();
  u16 i = 0;
  u16 l= 0;
  KEYDDR &= ~(1<<(minus));

for (l=0;l<schw+400;l++){
  if( KEYPIN & 1<<(minus) )
    i++;
}

  KEYPORT &= ~(1<<(minus));						
  KEYDDR |= 1<<(minus);                        	
  sei();

  return i;
}

u8 plattecap()
{
  cli();
  u16 i = 0;
  u16 l= 0;
  KEYDDR &= ~(1<<(platte));

for (l=0;l<schw+400;l++){
  if( KEYPIN & 1<<(platte) )
    i++;
}

  KEYPORT &= ~(1<<(platte));						
  KEYDDR |= 1<<(platte);                        	
  sei();

  return i;
}

u8 druckcap()
{
  cli();
  u16 i = 0;
  u16 l= 0;
  KEYDDR &= ~(1<<(druck));

for (l=0;l<schw+400;l++){
  if( KEYPIN & 1<<(druck) )
    i++;
}

  KEYPORT &= ~(1<<(druck));						
  KEYDDR |= 1<<(druck);                        	
  sei();

  return i;
}


u8 manuelcap()
{
  cli();
  u16 i = 0;
  u16 l= 0;
  KEYDDR &= ~(1<<(manuel));

for (l=0;l<schw+400;l++){
  if( KEYPIN & 1<<(manuel) )
    i++;
}

  KEYPORT &= ~(1<<(manuel));						
  KEYDDR |= 1<<(manuel);                        	
  sei();

  return i;
}

void boot()
{
	t1_init();
	LEDDDR=0xff;
	LEDPORT |= (1<<UVLEDS);
	LEDPORT |= (1<<LED0);
	LEDPORT |= (1<<LED1);
	LEDPORT |= (1<<LED2);
	LEDPORT |= (1<<LED3);	
	sei();

	}

int main( void )
{
	boot();
	for (;;) {
		if (abbruchcap() > schw){
			zeit=0;
			status=0;
		}
		if (zeit<65535 && status !=4)
			if (pluscap() > schw){
				zeit=zeit+dyn*korr;
				LEDPORT &= ~(1<<LED3);
				_delay_ms(200);
				LEDPORT |= (1<<LED3);
				}
		if (zeit>dyn*korr)
			if (minuscap() > schw){
				zeit=zeit-dyn*korr;
				LEDPORT &= ~(1<<LED3);
				_delay_ms(200);
				LEDPORT |= (1<<LED3);
				}
		if (status == 0){
			if (plattecap() > schw){ 
				zeit=plattezeit*korr;
				status=1;
				LEDPORT &= ~(1<<LED0);
				}
			if (lackcap() > schw){
				zeit=lackzeit*korr;
				LEDPORT &= ~(1<<LED1);
				status=2;
				}
			if (druckcap() > schw){
				zeit=druckzeit*korr;
				LEDPORT &= ~(1<<LED2);
				status=3;
				}
			if (manuelcap() > schw){
				status=4;
				LEDPORT &= ~(1<<LED3);
				zeit=65535;
				}
		}
	if (zeit !=0)
		LEDPORT &= ~(1<<UVLEDS);
	
	
	
	}
}

ISR(TIMER0_OVF_vect)
{
	if(zeit>0){
		zeit--;

	}else{
		LEDPORT |= (1<<UVLEDS);
		LEDPORT |= (1<<LED0);
		LEDPORT |= (1<<LED1);
		LEDPORT |= (1<<LED2);
		LEDPORT |= (1<<LED3);
		status=0;
	}

	//TCNT0=0;

}

